﻿using Hotline.Caching.Interfaces;
using Hotline.Caching.Services;
using Hotline.CallCenter.Calls;
using Hotline.Orders;
using Hotline.Push.FWMessage;
using Hotline.Repository.SqlSugar.DataPermissions;
using Hotline.Settings;
using Hotline.Share.Dtos.Push;
using Hotline.Share.Enums.Order;
using Hotline.Share.Tools;
using Microsoft.Extensions.Logging;
using SqlSugar;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;
using XF.Domain.Repository;
using XF.Domain.Repository.Extensions;
using static System.Runtime.InteropServices.JavaScript.JSType;

namespace Hotline.Repository.SqlSugar.Orders;
public class OrderVisitRepository : BaseRepository<OrderVisit>, IOrderVisitRepository, IScopeDependency
{
    private readonly IRepository<OrderVisitDetail> _orderVisitDetailRepository;
    private readonly ILogger<OrderVisitRepository> _logger;
    private readonly IRepository<Order> _orderRepository;
    private readonly ISystemDicDataCacheManager _systemDicDataCacheManager;

    public OrderVisitRepository(ISugarUnitOfWork<HotlineDbContext> uow, IDataPermissionFilterBuilder dataPermissionFilterBuilder, IRepository<OrderVisitDetail> orderVisitDetailRepository, ILogger<OrderVisitRepository> logger, IRepository<Order> orderRepository, ISystemDicDataCacheManager systemDicDataCacheManager) : base(uow, dataPermissionFilterBuilder)
    {
        _orderVisitDetailRepository = orderVisitDetailRepository;
        _logger = logger;
        _orderRepository = orderRepository;
        _systemDicDataCacheManager = systemDicDataCacheManager;
    }

    /// <summary>
    /// 用户回访短信回复更新回访状态
    /// </summary>
    /// <param name="dto"></param>
    /// <param name="data"></param>
    /// <returns></returns>
    public async Task UpdateSmsReplyAsync(PushReceiveMessageDto dto, Message data)
    {
        _logger.LogInformation($"UpdateSmsReplyAsync 收到通知: {dto.ToJson()}");
        if (data.IsSmsReply == false || data.SmsReplyContent.IsNullOrEmpty() || data.ExternalId.IsNullOrEmpty()) return;

        var orderVisit = await GetAsync(data.ExternalId)
             ?? throw new UserFriendlyException($"回访单不存在, visitId: {data.ExternalId} message: {data.ToJson()}");

        if (orderVisit.VisitState == EVisitState.Visited)
            throw new UserFriendlyException($"回访单已回访. visitId: {data.ExternalId}");

        var replyTxt = data.SmsReplyContent.Trim();
        var result = string.Empty;
        if (SettingConstants.ReplyToEnumMap.TryGetValue(replyTxt, out result) == false)
        {
            var m = SettingConstants.ReplyToEnumMap.FirstOrDefault(item => item.Value.StartsWith(replyTxt));
            replyTxt = m.Key;
            result = m.Value;
        }

        if (result.IsNullOrEmpty()) throw new UserFriendlyException($"用户回复短信内容异常; reply: {data.SmsReplyContent}");
        var replySplit = result.Split("|");
        if (new string[] { "4", "5" }.Contains(replyTxt))
        {
            // “短信不满意待回访”状态下，由其他方式再次进行回访，回访方式需更新为最新的回访方式
            // 故在此置为空
            orderVisit.VisitType = null;
        }
        var visitSatisfaction = _systemDicDataCacheManager.GetSysDicDataCache(SysDicTypeConsts.VisitSatisfaction)
            .First(m => m.DicDataValue == replySplit[4]);
        orderVisit.NowEvaluate = new Share.Dtos.Kv(visitSatisfaction.DicDataValue, visitSatisfaction.DicDataName);

        orderVisit.VisitTime = DateTime.Now;
        orderVisit.VisitState = replySplit[1].ToEnum<EVisitState>();
        await UpdateAsync(orderVisit, ignoreNullColumns: false);

        if (orderVisit.VisitState == EVisitState.Visited)
        {
            await _orderRepository.GetAsync(orderVisit.OrderId)
                .Then(async order =>
                {
                    order.Visited(replyTxt, replySplit[0]);
                    await _orderRepository.UpdateAsync(order);
                });

        }
        var detailOrg = await _orderVisitDetailRepository.Queryable()
            .Where(m => m.VisitId == orderVisit.Id && m.VisitTarget == EVisitTarget.Org)
            .ToListAsync();
        foreach (var item in detailOrg)
        {
            item.OrgProcessingResults = new Share.Dtos.Kv(visitSatisfaction.DicDataValue, visitSatisfaction.DicDataName);
            item.OrgHandledAttitude = new Share.Dtos.Kv(visitSatisfaction.DicDataValue, visitSatisfaction.DicDataName);
            await _orderVisitDetailRepository.UpdateAsync(item);
        }

        await _orderVisitDetailRepository.Queryable()
            .Where(m => m.VisitId == orderVisit.Id && m.VisitTarget == EVisitTarget.Seat)
            .FirstAsync()
            .Then(async detailSeat =>
            {
                detailSeat.SeatEvaluate ??= replySplit[2].ToEnum<ESeatEvaluate>();
                detailSeat.VoiceEvaluate ??= replySplit[3].ToEnum<EVoiceEvaluate>();
                await _orderVisitDetailRepository.UpdateAsync(detailSeat);
            });
    }
}
